cd c:/Python/standard/standard1
# standard1.py
#-------------------------------------------------------------------
# Pythonの標準操作について学ぶ（その1）
# 
#-------------------------------------------------------------------
# 2021/02/19:	新規作成
#-------------------------------------------------------------------
#<入力
#< iryou1.csv：		サンプル医療データ
#>出力ファイル
# 中間ファイル
#> iryou1_2_leave.csv：	退院日が2回の患者を確認するファイル
#> iryou1_d.csv：	分析用のデータを確認するファイル
# 結果ファイル
#> iryou1_d_corr.csv：	分析対象項目の相関行列
#> ward_9mon_count.csv：	9月の評価日と病棟のクロス表
#> ward_9mon_count.png：	9月の評価日別の病棟別の棒グラフ
#> ward1000_9mon_mean.csv：	病棟ID1000の9月の評価項目の平均値
#> ward1000_9mon_mean.png：	病棟ID1000の9月の評価項目別平均値の折れ線グラフ
#> ward1000_9mon_mean_values.png：	病棟ID1000の9月の評価項目別平均値の折れ線グラフ（各項目）
#> ward1000_9mon_mean_score.png：	病棟ID1000の9月の評価項目別平均値の折れ線グラフ（合計）
#> enter_n_count.csv：	入院経過日と病棟のクロス表
#> ward_enter_count.png：	入院経過日の病棟別棒グラフ
#> ward1000_e30_mean_values.png：	入院経過日別の評価項目別平均値の折れ線グラフ（各項目）
#> ward1000_e30_mean_score.png：	入院経過日別の評価項目別平均値の折れ線グラフ（合計）
#> ward1000_e30_mean.csv：	入院経過日別の評価項目の平均値
#> ward1000_e30_mean.png：	入院経過日別の評価項目別平均値の折れ線グラフ
#> ward1000_e30_count_mean.png：	入院日別の入院患者数グラフと項目別平均値の合わせグラフ
#> leave_n_count.csv：	退院予定日までの日数と病棟のクロス表
#> ward_leave_count.png：	退院予定日別の病棟別棒グラフ
#> ward1000_l30_mean.csv：	退院予定日までの日別の評価項目の平均値
#> ward1000_l30_mean_values.png：	退院予定日別の評価項目別平均値の折れ線グラフ（各項目）
#> ward1000_l30_mean_score.png：	退院予定日別の評価項目別平均値の折れ線グラフ（合計）
#> ward1000_l30_count_mean.png：	退院予定日別の入院患者数グラフと項目別平均値の合わせグラフ
# 最終保存ファイル
#> iryou1_data.csv：	入院経過、退院予定を付加（csv）
#> iryou1_data.pickle：	入院経過、退院予定を付加（pickle）
#
#-------------------------------------------------------------------
# 参照ライブラリ
import pandas as pd
import numpy as np
from datetime import datetime,date # 日時関連ライブラリ
import matplotlib.pyplot as plt # 図形を描くための基本ライブラリ
import seaborn as sns # seabornという図形描画のライブラリ

#-------------------------------------------------------------------
# サンプル医療データの読み込み
#-------------------------------------------------------------------
IN_FILE = "data/iryou1.csv" 
in_type = {"ward_id":"object","pat_id":"object",
		   "leave_date":"object","enter_date":"object","assess_date":"object",
		   "j1":"int","j2":"int","k1":"int","k2":"int","k3":"int",
		   "s1":"int","s2":"int","i1":"int","i2":"int","score":"int"}
%time iryou1 = pd.read_csv(IN_FILE,header=0,dtype=in_type,index_col=False,encoding="cp932")
# 46.9ms
len(iryou1) # データ件数
# 18491
iryou1.head(5).T # 最初の5レコードを表示（.T：転置）

#-------------------------------------------------------------------
# データの確認　・・・　テストデータであり、実際の入院患者の状況とは一致しない
#-------------------------------------------------------------------
# 病棟IDの状況
iryou1["ward_id"].value_counts().sort_index() # 病棟別データ件数
# 1000	4251
# 2000	5456
# 3000	4148
# 4000	4636
4251 + 5456 + 4148 + 4636
# 18491件： 欠損値(NaN)が無い
# その他の文字情報の状況
iryou1["pat_id"].describe()
# count  18491	要素のデータ件数
# unique  1688	ユニークな（一意な）値の要素の個数
# top 	  0105	最頻値（mode）
# freq      91	最頻値の頻度（出現回数）
iryou1["pat_id"].isnull().sum() # 欠損値の数をカウント
# 0
iryou1["leave_date"].describe()
# count  15741	<= 欠損値ではない件数
# unique    91	9月1日～11月30日
# top 20201027	2020年10月27日
# freq     391	4病棟の合計退院が最も多かった件数
iryou1["leave_date"].isnull().sum() # 欠損値の数をカウント
# 2750
15741 + 2750
# 18491
iryou1["enter_date"].describe()
# count  18491
# unique   149	(>=91)9月1日以前の入院日が存在する
# top 20200927	2020年9月27日
# freq     455	4病棟の合計入院が最も多かった件数
iryou1["assess_date"].describe()
# count  18491
# unique    91	9月1日～11月30日
# top 20200901	2020年9月1日
# freq     262	4病棟の合計評価が最も多かった件数
18491 / 4 / 91
# 50.799・・・　＜＝病棟の1日の平均評価データ件数

# 残りの整数情報の状況
iryou1["j1"].describe()
# count  18491.000000	要素のデータ件数
# mean       2.866476	算術平均
# std        2.375617	標準偏差
# min        0.000000	最小値
# 25%        1.000000	1/4分位
# 50%        4.000000	中央値
# 75%        4.000000	3/4分位数
# MAX        9.000000	最大値
iryou1["j2"].describe()
# count  18491.000000
# mean       4.159645
# std        2.479649
# min        0.000000
# 25%        2.000000
# 50%        5.000000
# 75%        6.000000
# MAX        9.000000
iryou1["k1"].describe()
# count  18491.000000
# mean       3.553837
# std        2.424313
# min        0.000000
# 25%        2.000000
# 50%        3.000000
# 75%        5.000000
# MAX        9.000000
iryou1["k2"].describe()
# count  18491.000000
# mean       3.814288
# std        2.229167
# min        0.000000
# 25%        2.000000
# 50%        4.000000
# 75%        6.000000
# MAX        7.000000
iryou1["k3"].describe()
# count  18491.000000
# mean       4.029312
# std        2.640043
# min        0.000000
# 25%        2.000000
# 50%        4.000000
# 75%        6.000000
# MAX        9.000000
iryou1["s1"].describe()
# count  18491.000000
# mean       2.471851
# std        1.846665
# min        0.000000
# 25%        1.000000
# 50%        2.000000
# 75%        4.000000
# MAX        7.000000
iryou1["s2"].describe()
# count  18491.000000
# mean       2.838895
# std        2.038158
# min        0.000000
# 25%        1.000000
# 50%        3.000000
# 75%        4.000000
# MAX        7.000000
iryou1["i1"].describe()
# count  18491.000000
# mean       2.135579
# std        1.567951
# min        0.000000
# 25%        1.000000
# 50%        2.000000
# 75%        3.000000
# MAX        7.000000
iryou1["i2"].describe()
# count  18491.000000
# mean       2.336434
# std        1.727862
# min        0.000000
# 25%        1.000000
# 50%        2.000000
# 75%        4.000000
# MAX        7.000000
iryou1["score"].describe()
# count  18491.000000
# mean      28.206317
# std        9.021715
# min        4.000000
# 25%       21.000000
# 50%       28.000000
# 75%       35.000000
# MAX       61.000000

# 患者ID別の入院日の状況
len(iryou1["pat_id"].unique()) # 患者IDの種類数
# 1686
pat_id_n_enter = iryou1.groupby("pat_id")["enter_date"].unique().apply(len) # 患者別入院回数
pat_id_n_enter.value_counts().sort_index() # 入院回数の分布
# 1  1493	入院1回の患者数
# 2   158	入院2回の患者数
# 3    31	入院3回の患者数
# 4     4	入院4回の患者数
1493 + 158 + 31 + 4
# 1686

#-------------------------------------------------------------------
# 退院日が欠損値のケースを修正
#-------------------------------------------------------------------
# 入院患者（患者ID＋入院日）の評価日データに並び替え
iryou1["pat_enter"] = iryou1["pat_id"] + iryou1["enter_date"] # 入院患者のユニークIDを作成
iryou1 = iryou1.sort_values(["pat_enter","assess_date"]) # 入院患者別の評価日別に並び替え
len(iryou1["pat_enter"].unique()) # 入院患者の種類数（患者IDが複数回入院する為、多くなる）
# 1918
# 入院患者別の退院日の状況を確認
pat_enter_leave = iryou1.groupby("pat_enter")["leave_date"].unique() # 入院患者毎のユニークな退院日
pat_enter_leave # 退院日が無い（未退院）のデータと、退院日が後から付加された場合がある事が分かる
# 1918
pat_enter_n_leave = pat_enter_leave.apply(len) # 入院患者毎の退院日数
pat_enter_n_leave.value_counts() # 退院日数毎の入院患者数
# 1   1631 <= 退院日が1回ある入院患者数　＋　退院日が無い未退院の入院患者数
# 2    287 <= 退院日が2種類存在する入院患者数
1631 + 287
# 1918
pat_enter_n_leave = pd.DataFrame(pat_enter_n_leave) # Seriees型を DataFrameに変換
pat_enter_n_leave # 確認する
# 1918
pat_enter_n_leave = pat_enter_n_leave.rename(columns={"leave_date":"n_leave"}) # 列名の変更
pat_enter_n_leave # 確認する
# 1918
iryou1_n_leave = pd.merge(iryou1,pat_enter_n_leave,left_on="pat_enter",right_index=True,how="left")
iryou1_2_leave = iryou1_n_leave[iryou1_n_leave["n_leave"] == 2] # 退院日数が2回の入院患者のデータを抽出
iryou1_2_leave # 確認する
# 6083 × 17
iryou1_2_leave.to_csv("iryou1_2_leave.csv") # 全てを確認するため、csvファイルに書き出す
# iryou1_2_leave.csvの中身を確認する
# 同一入院患者で、退院日が無い日と退院日が存在するケースは、
# 退院日が存在するのは、退院日を含む評価月の評価日のデータであって、
# 退院日の以前の評価月の評価日のデータには、退院日が設定されていないことが分かる
# 医療データは、月単位で作成されているため、退院前の入院付きのデータには、
# 退院日が設定されていないからだと考えると理解できる
# つまり、退院日前の評価月のレコードにも、退院月の退院日情報を補完する必要がある
# 退院日の存在するデータを作成
iryou1_leave = iryou1.dropna(subset=["leave_date"]) # 退院日が無いデータは削除
iryou1_leave # 退院日が存在するデータだけになった
# 12345 × 16
iryou1_leave = iryou1_leave[["pat_enter","leave_date"]] # 入院患者IDと退院日だけのデータを作成
len(iryou1_leave["pat_enter"].unique()) # ユニークな入院患者ID数
# 1705
iryou1_leave.groupby("pat_enter")["leave_date"].unique().apply(len).value_counts() # 入院患者ID毎の退院日の種類数毎の入院患者ID数
# 1		1705 <= 退院日の存在する入院患者IDの退院日は、1種類だけであった
iryou1_leave = iryou1_leave.drop_duplicates(keep="last") # 重複しているデータの最後のレコードを抽出
iryou1_leave # 入院患者ID別の退院日を設定
# 1705
iryou1 = iryou1.rename(columns={"leave_date":"leave_old"}) # 列名の変更
iryou1 # 名前が変わったことを確認
# 18491 × 16
iryou1 = pd.merge(iryou1,iryou1_leave,on="pat_enter",how="left") # 正しい退院日を追加
iryou1 # 正しい退院日（退院日が無い入院患者はNaN）が設定されている
# 18491 × 17
iryou1[iryou1["leave_old"] != iryou1["leave_date"]][["leave_old","leave_date"]].dropna(subset=["leave_date"])
# 34725 <= 退院日が欠損値だったレコードに退院日が設定された事を確認
# これで、未退院患者以外で、退院日の欠損値は無くなった
# 前の退院日数毎の入院患者数の確認処理を再実行して確認
pat_enter_leave = iryou1.groupby("pat_enter")["leave_date"].unique() # 入院患者毎のユニークな退院日
pat_enter_leave # 退院日が無い（未退院）のデータと、退院日が後から付加された場合がある事が分かる
# 1918
pat_enter_n_leave = pat_enter_leave.apply(len) # 入院患者毎の退院日数
pat_enter_n_leave.value_counts() # 退院日数毎の入院患者数
# 1   1918 <= 退院日が1回ある入院患者数　＋　退院日が無い未退院の入院患者数
# 退院日が2種類存在する入院患者数は、無くなった
#-------------------------------------------------------------------
iryou1["enter_d"] = pd.to_datetime(iryou1["enter_date"],format='%Y%m%d') # 文字列を日付に変換
iryou1["leave_d"] = pd.to_datetime(iryou1["leave_date"],format='%Y%m%d') # 文字列を日付に変換
iryou1["assess_d"] = pd.to_datetime(iryou1["assess_date"],format='%Y%m%d') # 文字列を日付に変換
iryou1[["enter_d","leave_d","assess_d"]] # 日付データの確認
iryou1.head(3).T
iryou1_d = iryou1[["ward_id","pat_enter","pat_id",
				   "enter_d","leave_d","assess_d","enter_date","leave_date","assess_date",
		           "j1","j2","k1","k2","k3","s1","s2","i1","i2","score"]] # 分析に使用する項目の選択
iryou1_d.head(3).T
# 日付を年月日から月日に変換
iryou1_d["enter_date"] = iryou1_d["enter_date"].apply(lambda x: x[4:9])
iryou1_d["leave_date"] = iryou1_d["leave_date"].apply(lambda x: x[4:9] if pd.notnull(x) else np.nan)
iryou1_d["assess_date"] = iryou1_d["assess_date"].apply(lambda x: x[4:9])
iryou1_d
# 18491 × 19
iryou1_d.head(3).T # 月日の変換された事を確認
iryou1_d.to_csv("iryou1_d.csv") # 分析用のデータをエディタで確認

#-------------------------------------------------------------------
# データ分析
#-------------------------------------------------------------------
# 分析用の前処理
# 病棟ID別の分析を容易にするために病棟ID毎に並べ替え
iryou1_d = iryou1_d.sort_values(["ward_id","pat_enter","assess_d"])
iryou1_d # Indexの値が並べ替えで変わった事を確認
# 18491 × 19
iryou1_data = iryou1_d.copy()
iryou1_data["assess_mon"] = iryou1_data["assess_d"].apply(lambda x: x.month)
iryou1_data["assess_mon"].value_counts().sort_index()
#  9	6403
# 10	5873
# 11	6215
6403 + 5873 + 6215
# 18491
# 月別データの作成
iryou1_9mon = iryou1_data[iryou1_data["assess_mon"] == 9]
iryou1_10mon = iryou1_data[iryou1_data["assess_mon"] == 10]
iryou1_11mon = iryou1_data[iryou1_data["assess_mon"] == 11]
iryou1_9mon = iryou1_9mon.drop("assess_mon",axis=1)
iryou1_10mon = iryou1_10mon.drop("assess_mon",axis=1)
iryou1_11mon = iryou1_11mon.drop("assess_mon",axis=1)
# 病棟別の月別のデータを作成
ward1000_9mon = iryou1_9mon[iryou1_9mon["ward_id"] == "1000"]
ward2000_9mon = iryou1_9mon[iryou1_9mon["ward_id"] == "2000"]
ward3000_9mon = iryou1_9mon[iryou1_9mon["ward_id"] == "3000"]
ward4000_9mon = iryou1_9mon[iryou1_9mon["ward_id"] == "4000"]
ward1000_10mon = iryou1_10mon[iryou1_10mon["ward_id"] == "1000"]
ward2000_10mon = iryou1_10mon[iryou1_10mon["ward_id"] == "2000"]
ward3000_10mon = iryou1_10mon[iryou1_10mon["ward_id"] == "3000"]
ward4000_10mon = iryou1_10mon[iryou1_10mon["ward_id"] == "4000"]
ward1000_11mon = iryou1_11mon[iryou1_11mon["ward_id"] == "1000"]
ward2000_11mon = iryou1_11mon[iryou1_11mon["ward_id"] == "2000"]
ward3000_11mon = iryou1_11mon[iryou1_11mon["ward_id"] == "3000"]
ward4000_11mon = iryou1_11mon[iryou1_11mon["ward_id"] == "4000"]
#-------------------------------------------------------------------
# グラフ処理の為の処理
# seabornのバージョンを確認
sns.__version__
# '0.11.0' <= seabornのバージョンを確認
# IPythonを修了した状態で、以下を実行すると全パッケージのバージョンが確認できる
# conda list

# グラフを処理する場合、以下のコマンドによりインターラクティブモードにすると表示を自動化できる
plt.ion()  # IPython処理中に、一度だけ実行
#-------------------------------------------------------------------
# 分析対象のデータ項目の相関関係
iryou1_d_corr = iryou1_d.corr() # 数値項目だけの相互の相関係数を計算
iryou1_d_corr
iryou1_d_corr.to_csv("iryou1_d_corr.csv")
# j2,k1,k2,k3の項目がそれぞれ相関が比較的高く、score（合計）との相関が高い事が分かる

#-------------------------------------------------------------------
# 評価日毎の分析
# （一部の分析をサンプル的に取り上げて実施する）
#-------------------------------------------------------------------
# 9月の病棟別の件数について確認する
# 件数の確認（クロス表）
ward_9mon_count = pd.crosstab(iryou1_9mon["assess_date"],iryou1_9mon["ward_id"],margins=True)
ward_9mon_count
ward_9mon_count.to_csv("ward_9mon_count.csv")

# グラフで確認
g = plt.figure(figsize=(12.0,7.0))
g = sns.countplot(x="assess_date",hue="ward_id",data=iryou1_9mon) # 9月の病棟別の件数グラフ
# 間を空けて実行しないと、次のX軸ラベルの処理が正常処理できないので注意
g.set_xticklabels(g.get_xticklabels(),rotation=90)  # X軸のラベルを90度傾ける
g = plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=0.5,fontsize=12)  # 凡例を右横上に配置
plt.savefig("ward_9mon_count.png")  # ファイル保存
plt.close()  # グラフの終了

#-------------------------------------------------------------------
# 平均値の状況
# 病棟ID＝1000、9月のデータについて分析する
ward1000_9mon.describe() # 全ての量的変数の要約統計量を確認
# 選択したデータに対する平均値、標準偏差、各4分位の値が確認できた
# 
ward1000_9mon_mean = ward1000_9mon.groupby(["assess_date"]).mean() # 評価日毎に集計し平均値を求める
ward1000_9mon_mean
ward1000_9mon_mean.to_csv("ward1000_9mon_mean.csv")

# グラフで確認
g = plt.figure(figsize=(12.0,7.0))
g = sns.lineplot(data=ward1000_9mon_mean)
# 間を空けて実行しないと、次のX軸ラベルの処理が正常処理できないので注意
g.set_xticklabels(g.get_xticklabels(),rotation=45)  # X軸のラベルを45度傾ける
g = plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=0.5,fontsize=12)  # 凡例を右横上に配置
plt.savefig("ward1000_9mon_mean.png")  # ファイル保存
plt.close()  # グラフの終了

# score値は、他の数値の合計値であり、データ範囲が異なるため。グラフを分けて表示する
ward1000_9mon_mean_values = ward1000_9mon_mean[["j1","j2","k1","k2","k3","s1","s2","i1","i2"]]
ward1000_9mon_mean_score = ward1000_9mon_mean["score"]

g = plt.figure(figsize=(12.0,7.0))
g = sns.lineplot(data=ward1000_9mon_mean_values)
# 間を空けて実行しないと、次のX軸ラベルの処理が正常処理できないので注意
g.set_xticklabels(g.get_xticklabels(),rotation=45)  # X軸のラベルを45度傾ける
g = plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=0.5,fontsize=12)  # 凡例を右横上に配置
plt.savefig("ward1000_9mon_mean_values.png")  # ファイル保存
plt.close()  # グラフの終了

g = plt.figure(figsize=(12.0,7.0))
g = sns.lineplot(data=ward1000_9mon_mean_score)
# 間を空けて実行しないと、次のX軸ラベルの処理が正常処理できないので注意
g.set_xticklabels(g.get_xticklabels(),rotation=45)  # X軸のラベルを45度傾ける
plt.savefig("ward1000_9mon_mean_score.png")  # ファイル保存
plt.close()  # グラフの終了

#-------------------------------------------------------------------
# 入院経過による分析
#-------------------------------------------------------------------
# 日付形式データによって、入院日からの評価日までの日数を計算
iryou1_d["enter_n"] = iryou1_d["assess_d"] - iryou1_d["enter_d"]
iryou1_d
# 18491
iryou1_d["enter_n"] = iryou1_d["enter_n"].apply(lambda x: x.days + 1) # 入院1日目を1とし、整数で扱う
iryou1_d[["assess_d","enter_d","enter_n"]]
# 18491
iryou1_d["enter_n"].value_counts().sort_index()
# 最大245日目の入院患者が居る
iryou1_e30 = iryou1_d[iryou1_d["enter_n"] <= 30] # 入院経過日数を30日までに制限
iryou1_e30
# 15947
# 入院経過日数別の病棟別の件数について確認する
# 件数の確認（クロス表）
enter_n_count = pd.crosstab(iryou1_e30["enter_n"],iryou1_e30["ward_id"],margins=True)
enter_n_count # 入院経過日が経過するに従って、入院患者数が減っていく
enter_n_count.to_csv("enter_n_count.csv")

# グラフで確認
g = plt.figure(figsize=(12.0,7.0))
g = sns.countplot(x="enter_n",hue="ward_id",data=iryou1_e30) # 入院経過の病棟別の件数グラフ
plt.savefig("ward_enter_count.png")  # ファイル保存
plt.close()  # グラフの終了

#-------------------------------------------------------------------
# 病棟別の平均値の状況
ward1000_e30 = iryou1_e30[iryou1_e30["ward_id"] == "1000"]
ward2000_e30 = iryou1_e30[iryou1_e30["ward_id"] == "2000"]
ward3000_e30 = iryou1_e30[iryou1_e30["ward_id"] == "3000"]
ward4000_e30 = iryou1_e30[iryou1_e30["ward_id"] == "4000"]
# 病棟ID＝1000の入院経過データについて分析する
ward1000_e30
# 3877
ward1000_e30_mean = ward1000_e30.groupby(["enter_n"]).mean()
ward1000_e30_mean
ward1000_e30_mean.to_csv("ward1000_e30_mean.csv")

# グラフで確認
g = plt.figure(figsize=(12.0,7.0))
g = sns.lineplot(data=ward1000_e30_mean)
g = plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=0.5,fontsize=12)  # 凡例を右横上に配置
plt.savefig("ward1000_e30_mean.png")  # ファイル保存
plt.close()  # グラフの終了

# score値は、他の数値の合計値であり、データ範囲が異なるため。グラフを分けて表示する
ward1000_e30_mean_values = ward1000_e30_mean[["j1","j2","k1","k2","k3","s1","s2","i1","i2"]]
ward1000_e30_mean_score = ward1000_e30_mean["score"]

g = plt.figure(figsize=(12.0,7.0))
g = sns.lineplot(data=ward1000_e30_mean_values)
g = plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=0.5,fontsize=12)  # 凡例を右横上に配置
plt.savefig("ward1000_e30_mean_values.png")  # ファイル保存
plt.close()  # グラフの終了

g = plt.figure(figsize=(12.0,7.0))
g = sns.lineplot(data=ward1000_e30_mean_score)
plt.savefig("ward1000_e30_mean_score.png")  # ファイル保存
plt.close()  # グラフの終了
# j2、k2、k3は、入院経過に従って値が上がっている。結果として合計値も入院経過に従って上がっている事が分かる

# ward=1000の入院経過日別の患者数と各項目の平均値を1つのグラフに表示する
g = plt.figure(figsize=(12.0,7.0))
g = sns.countplot(x="enter_n",data=ward1000_e30,color="green") # 入院患者数の棒グラフ
g = sns.lineplot(data=ward1000_e30_mean_values) # 各項目の平均値の折れ線グラフ
# 入院患者数の縮尺で各項目の平均値を表示すると平均値の違いが判別できない
plt.close()  # グラフの終了

# 2つのグラフを一緒に描く
g = plt.figure(figsize=(10.0,10.0))
g.add_subplot(2,1,1) # 行方向に2つ、列方向に1つのグラフで、1つ目に描画
sns.countplot(x="enter_n",data=ward1000_e30,color="green") # 入院患者数の棒グラフ
g.add_subplot(2,1,2) # 行方向に2つ、列方向に1つのグラフで、2つ目に描画
sns.lineplot(data=ward1000_e30_mean_values) # 各項目の平均値の折れ線グラフ
plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=0.5,fontsize=12)  # 凡例を右横上に配置
# 2つのグラフを一度に表示することで比較ができるようになった
plt.close()  # グラフの終了

# 更に、2つのグラフを1つのグラフ上に表示する
# 但し、各項目の平均値を別の縮尺にして表示する
g = plt.figure(figsize=(12.0,7.0))
ax1 = g.add_subplot(1,1,1) # 行方向に1つ、列方向に1つのグラフで、1つ目に描画
sns.countplot(x="enter_n",data=ward1000_e30,color="lightgreen") # 入院患者数の棒グラフ
ax2 = ax1.twinx() # X軸を共通化した2つのY軸を使用する指定（XYが逆の場合は、twiny()を使用）
# 次の描画は、2つ目のY軸に描かれる
sns.lineplot(data=ward1000_e30_mean_values) # 各項目の平均値の折れ線グラフ
plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=1.5,fontsize=12)  # 凡例を右横上に配置（右側にY軸が追加されているので調整）
# なぜか、2つのグラフのX軸がずれている
# lineplotが、0から始まる位置で描いている
# leave_nを文字列に変更してみる
ward1000_e30
ward1000_e30 = ward1000_e30.sort_values(["enter_n"]) # countplotで正しく並ばせるために
ward1000_e30["enter_n"] = ward1000_e30["enter_n"].astype('str').str.zfill(2) # 文字列が正しく並ぶように、前ゼロを付ける
ward1000_e30["enter_n"]
ward1000_e30_mean = ward1000_e30.groupby(["enter_n"]).mean()
ward1000_e30_mean_values = ward1000_e30_mean[["j1","j2","k1","k2","k3","s1","s2","i1","i2"]]
ward1000_e30_mean_score = ward1000_e30_mean["score"]

g = plt.figure(figsize=(12.0,7.0))
ax1 = g.add_subplot(1,1,1) # 行方向に1つ、列方向に1つのグラフで、1つ目に描画
sns.countplot(x="enter_n",data=ward1000_e30,color="lightgreen") # 入院患者数の棒グラフ
ax2 = ax1.twinx() # X軸を共通化した2つのY軸を使用する指定（XYが逆の場合は、twiny()を使用）
# 次の描画は、2つ目のY軸に描かれる
sns.lineplot(data=ward1000_e30_mean_values) # 各項目の平均値の折れ線グラフ
plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=1.5,fontsize=12)  # 凡例を右横上に配置（右側にY軸が追加されているので調整）
# X軸が一致した
# 入院が経過することで入院患者数が少なくなっており、平均値の母数が少なくなることから一定した値になり難いことに注意
plt.savefig("ward1000_e30_count_mean.png")  # ファイル保存
plt.close()  # グラフの終了

#-------------------------------------------------------------------
# 退院予定による分析
#-------------------------------------------------------------------
# 日付形式データによって、退院日からの評価日までの日数を計算
iryou1_d["leave_n"] = iryou1_d["leave_d"] - iryou1_d["assess_d"]
iryou1_d
# 18491 × 21
iryou1_d["leave_n"].isnull().sum() # 欠損値の数をカウント
# 2750
iryou1_d["leave_n"] = iryou1_d["leave_n"].apply(lambda x: x.days + 1) # 退院1日目を1とし、整数で扱う
iryou1_d[["assess_d","leave_d","leave_n"]]
# 18491
iryou1_d["leave_n"].value_counts().sort_index()
# 退院予定日数は、最大80日まで存在する
iryou1_l30 = iryou1_d[iryou1_d["leave_n"] <= 30] # 退院予定日数を30日までに制限
iryou1_l30
iryou1_l30 = iryou1_l30.drop(columns="enter_n")
# 14754 × 20
iryou1_l30["leave_n"].isnull().sum() # 欠損値の数をカウント
# 0 <= 欠損値は無い
iryou1_l30["leave_n"] = iryou1_l30["leave_n"].apply(int) # 欠損値が無いので、整数に変換
iryou1_l30["leave_n"].value_counts().sort_index()
# 退院予定日数が1から30までになった

# 退院予定日数別の病棟別の件数について確認する
# 件数の確認（クロス表）
leave_n_count = pd.crosstab(iryou1_l30["leave_n"],iryou1_l30["ward_id"],margins=True)
leave_n_count # 入院経過日が経過するに従って、入院患者数が減っていく
leave_n_count.to_csv("leave_n_count.csv")

# グラフで確認
g = plt.figure(figsize=(12.0,7.0))
g = sns.countplot(x="leave_n",hue="ward_id",data=iryou1_l30) # 入院経過の病棟別の件数グラフ
plt.savefig("ward_leave_count.png")  # ファイル保存
plt.close()  # グラフの終了

#-------------------------------------------------------------------
# 病棟別の平均値の状況
ward1000_l30 = iryou1_l30[iryou1_l30["ward_id"] == "1000"]
ward2000_l30 = iryou1_l30[iryou1_l30["ward_id"] == "2000"]
ward3000_l30 = iryou1_l30[iryou1_l30["ward_id"] == "3000"]
ward4000_l30 = iryou1_l30[iryou1_l30["ward_id"] == "4000"]
# 病棟ID＝1000の入院経過データについて分析する
ward1000_l30
ward1000_l30 = ward1000_l30.sort_values(["leave_n"]) # countplotで正しく並ばせるために
ward1000_l30["leave_n"] = ward1000_l30["leave_n"].astype('str').str.zfill(2) # 文字列が正しく並ぶように、前ゼロを付ける
ward1000_l30["leave_n"]
# 3503
ward1000_l30_mean = ward1000_l30.groupby(["leave_n"]).mean()
ward1000_l30_mean
ward1000_l30_mean.to_csv("ward1000_l30_mean.csv")

# グラフで確認
#g = plt.figure(figsize=(12.0,7.0))
#g = sns.lineplot(data=ward1000_l30_mean)
#g = plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=0.5,fontsize=12)  # 凡例を右横上に配置
#plt.savefig("ward1000_l30_mean.png")  # ファイル保存
#plt.close()  # グラフの終了

# score値は、他の数値の合計値であり、データ範囲が異なるため。グラフを分けて表示する
ward1000_l30_mean_values = ward1000_l30_mean[["j1","j2","k1","k2","k3","s1","s2","i1","i2"]]
ward1000_l30_mean_score = ward1000_l30_mean["score"]

g = plt.figure(figsize=(12.0,7.0))
g = sns.lineplot(data=ward1000_l30_mean_values)
g = plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=0.5,fontsize=12)  # 凡例を右横上に配置
plt.savefig("ward1000_l30_mean_values.png")  # ファイル保存
plt.close()  # グラフの終了

g = plt.figure(figsize=(12.0,7.0))
g = sns.lineplot(data=ward1000_l30_mean_score)
plt.savefig("ward1000_l30_mean_score.png")  # ファイル保存
plt.close()  # グラフの終了
# j2、k2、k3は、退院日に向かって下がっている。結果として合計値も退院日に向かって下がっている事が分かる
# この入院経過と退院予定の結果からは、入院後数値は上がっていくが、退院に向けて下がっていくと考えられる。
# 詳しく見てみると、入院日から3日程度は、k1以外は増加傾向が見られるが、その後は一定している。
# 20日経過頃から上がっている項目もあるが、入院患者数は少なくなっているので評価が難しい。
# 退院予定の方は、退院日の13日前から値が下がる傾向があり、合計では顕著に見て取れる。
# ＜※本データは、テストデータであり、実際の病院のデータではないことに留意する＞

# ward=1000の退院予定日の患者数と各項目の平均値を1つのグラフに表示する
#g = plt.figure(figsize=(12.0,7.0))
#g = sns.countplot(x="leave_n",data=ward1000_l30,color="green") # 入院患者数の棒グラフ
#g = sns.lineplot(data=ward1000_l30_mean_values) # 各項目の平均値の折れ線グラフ
# 入院患者数の縮尺で各項目の平均値を表示すると平均値の違いが判別できない
#plt.close()  # グラフの終了

# 2つのグラフを併せて描く
g = plt.figure(figsize=(10.0,10.0))
g.add_subplot(2,1,1) # 行方向に2つ、列方向に1つのグラフで、1つ目に描画
sns.countplot(x="leave_n",data=ward1000_l30,color="green") # 入院患者数の棒グラフ
g.add_subplot(2,1,2) # 行方向に2つ、列方向に1つのグラフで、2つ目に描画
sns.lineplot(data=ward1000_l30_mean_values) # 各項目の平均値の折れ線グラフ
plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=0.5,fontsize=12)  # 凡例を右横上に配置
# 2つのグラフを一度に表示することで比較ができるようになった
plt.close()  # グラフの終了

# 更に、2つのグラフを1つのグラフ上に表示する
# 但し、各項目の平均値を別の縮尺にして表示する
g = plt.figure(figsize=(12.0,7.0))
ax1 = g.add_subplot() # 行方向に1つ、列方向に1つのグラフで、1つ目に描画
sns.countplot(x="leave_n",data=ward1000_l30,color="lightgreen") # 入院患者数の棒グラフ
ax2 = ax1.twinx() # X軸を共通化した2つのY軸を使用する指定（XYが逆の場合は、twiny()を使用）
# 次の描画は、2つ目のY軸に描かれる
sns.lineplot(data=ward1000_l30_mean_values) # 各項目の平均値の折れ線グラフ
plt.legend(bbox_to_anchor=(1.0,1.0),loc="upper left",borderaxespad=1.5,fontsize=12)  # 凡例を右横上に配置（右側にY軸が追加されているので調整）
# 退院日から遡ることで入院患者数が少なくなっており、平均値の母数が少なくなることから一定した値になり難いことに注意
plt.savefig("ward1000_l30_count_mean.png")  # ファイル保存
plt.close()  # グラフの終了

#---------------------------------------------------------------------------------------------
# データ保存
#---------------------------------------------------------------------------------------------
iryou1_data = iryou1_d.copy()
# 18491 × 21
%time iryou1_data.to_csv("iryou1_data.csv",index=False) # indexを外してcsvファイルに書き出し
# 657ms
%time iryou1_data.to_pickle("iryou1ata_d.pickle") # pickleファイルに書き出し
# 53.4 ms
